@@ -28,17 +28,6 @@ void uwsgi_update_load_counters() {
 
 }
 
-void uwsgi_restore_auto_snapshot(int signum) {
-
-	if (uwsgi.workers[1].snapshot > 0) {
-		uwsgi.restore_snapshot = 1;
-	}
-	else {
-		uwsgi_log("[WARNING] no snapshot available\n");
-	}
-
-}
-
 void uwsgi_block_signal(int signum) {
 	sigset_t smask;
 	sigemptyset(&smask);
@@ -104,31 +93,6 @@ void uwsgi_master_manage_udp(int udp_fd) {
 	}
 }
 
-void uwsgi_master_restore_snapshot() {
-	int i, waitpid_status;
-	uwsgi_log("[snapshot] restoring workers...\n");
-	for (i = 1; i <= uwsgi.numproc; i++) {
-		if (uwsgi.workers[i].pid == 0)
-			continue;
-		kill(uwsgi.workers[i].pid, SIGKILL);
-		if (waitpid(uwsgi.workers[i].pid, &waitpid_status, 0) < 0) {
-			uwsgi_error("waitpid()");
-		}
-		if (uwsgi.auto_snapshot > 0 && i > uwsgi.auto_snapshot) {
-			uwsgi.workers[i].pid = 0;
-			uwsgi.workers[i].snapshot = 0;
-		}
-		else {
-			uwsgi.workers[i].pid = uwsgi.workers[i].snapshot;
-			uwsgi.workers[i].snapshot = 0;
-			kill(uwsgi.workers[i].pid, SIGURG);
-			uwsgi_log("Restored uWSGI worker %d (pid: %d)\n", i, (int) uwsgi.workers[i].pid);
-		}
-	}
-
-	uwsgi.restore_snapshot = 0;
-}
-
 void suspend_resume_them_all(int signum) {
 
 	int i;
@@ -326,9 +290,6 @@ int master_loop(char **argv, char **environ) {
 	}
 	uwsgi_unix_signal(SIGINT, kill_them_all);
 	uwsgi_unix_signal(SIGUSR1, stats);
-	if (uwsgi.auto_snapshot) {
-		uwsgi_unix_signal(SIGURG, uwsgi_restore_auto_snapshot);
-	}
 
 	atexit(uwsgi_master_cleanup_hooks);
 
@@ -514,6 +475,28 @@ int master_loop(char **argv, char **environ) {
 	// fsmon
 	uwsgi_fsmon_setup();
 
+	uwsgi_foreach(usl, uwsgi.signal_timers) {
+		char *space = strchr(usl->value, ' ');
+		if (!space) {
+			uwsgi_log("invalid signal timer syntax, must be: <signal> <seconds>\n");
+			exit(1);
+		}
+		*space = 0;
+		uwsgi_add_timer(atoi(usl->value), atoi(space+1));
+		*space = ' ';
+	}
+
+	uwsgi_foreach(usl, uwsgi.rb_signal_timers) {
+                char *space = strchr(usl->value, ' ');
+                if (!space) {
+                        uwsgi_log("invalid redblack signal timer syntax, must be: <signal> <seconds>\n");
+                        exit(1);
+                }
+                *space = 0;
+                uwsgi_signal_add_rb_timer(atoi(usl->value), atoi(space+1), 0);
+                *space = ' ';
+        }
+
 	// setup cheaper algos (can be stacked)
 	uwsgi.cheaper_algo = uwsgi_cheaper_algo_spare;
 	if (uwsgi.requested_cheaper_algo) {
@@ -567,21 +550,6 @@ int master_loop(char **argv, char **environ) {
 		// check for daemons (smart and dumb)
 		uwsgi_daemons_smart_check();
 
-
-		if (uwsgi.respawn_snapshots) {
-			for (i = 1; i <= uwsgi.respawn_snapshots; i++) {
-				if (uwsgi_respawn_worker(i))
-					return 0;
-			}
-
-			uwsgi.respawn_snapshots = 0;
-		}
-
-		if (uwsgi.restore_snapshot) {
-			uwsgi_master_restore_snapshot();
-			continue;
-		}
-
 		// cheaper management
 		if (uwsgi.cheaper && !uwsgi.status.is_cheap && !uwsgi_instance_is_reloading && !uwsgi_instance_is_dying && !uwsgi.workers[0].suspended) {
 			if (!uwsgi_calc_cheaper())
