@@ -498,6 +498,70 @@ XS(XS_websocket_send) {
 	XSRETURN_UNDEF;
 }
 
+XS(XS_websocket_send_from_sharedarea) {
+        dXSARGS;
+
+        psgi_check_args(2);
+	int id = SvIV(ST(0));
+        uint64_t pos = SvIV(ST(1));
+	uint64_t len = 0;
+
+	if (items > 2) {
+		len = SvIV(ST(2));
+	}
+	
+        struct wsgi_request *wsgi_req = current_wsgi_req();
+
+        if (uwsgi_websocket_send_from_sharedarea(wsgi_req, id, pos, len)) {
+                croak("unable to send websocket message from sharedarea");
+        }
+
+        XSRETURN_UNDEF;
+}
+
+
+XS(XS_websocket_send_binary) {
+        dXSARGS;
+
+        char *message = NULL;
+        STRLEN message_len = 0;
+
+        psgi_check_args(1);
+
+        message = SvPV(ST(0), message_len);
+
+        struct wsgi_request *wsgi_req = current_wsgi_req();
+
+        if (uwsgi_websocket_send_binary(wsgi_req, message, message_len)) {
+                croak("unable to send websocket binary message");
+        }
+
+        XSRETURN_UNDEF;
+}
+
+XS(XS_websocket_send_binary_from_sharedarea) {
+        dXSARGS;
+
+        psgi_check_args(2);
+        int id = SvIV(ST(0));
+        uint64_t pos = SvIV(ST(1));
+        uint64_t len = 0;
+
+        if (items > 2) {
+                len = SvIV(ST(2));
+        }
+
+        struct wsgi_request *wsgi_req = current_wsgi_req();
+
+        if (uwsgi_websocket_send_binary_from_sharedarea(wsgi_req, id, pos, len)) {
+                croak("unable to send websocket binary message from sharedarea");
+        }
+
+        XSRETURN_UNDEF;
+}
+
+
+
 XS(XS_websocket_recv) {
 	dXSARGS;
 
@@ -668,6 +732,106 @@ XS(XS_metric_get) {
         XSRETURN(1);
 }
 
+XS(XS_sharedarea_wait) {
+        dXSARGS;
+        int id;
+	int freq = 0;
+	int timeout = 0;
+
+	psgi_check_args(1);
+	
+	id = SvIV(ST(0));
+	if (items > 1) {
+		freq = SvIV(ST(1));
+	}
+
+	if (uwsgi_sharedarea_wait(id, freq, timeout)) {
+                croak("unable to wait for sharedarea %d", id);
+                XSRETURN_UNDEF;
+        }
+	XSRETURN_YES;
+}
+
+XS(XS_sharedarea_read) {
+	dXSARGS;
+	int id;
+	uint64_t pos;
+	uint64_t len = 0;
+	psgi_check_args(2);	
+
+	id = SvIV(ST(0));
+	pos = SvIV(ST(1));
+
+	if (items > 2) {
+		len = SvIV(ST(2));	
+	}
+	else {
+		struct uwsgi_sharedarea *sa = uwsgi_sharedarea_get_by_id(id, pos);
+		if (!sa) {
+			croak("unable to read from sharedarea %d", id);
+                	XSRETURN_UNDEF;
+		}
+		len = (sa->max_pos+1)-pos;
+	}
+
+	char *buf = uwsgi_malloc(len);
+	int64_t rlen = uwsgi_sharedarea_read(id, pos, buf, len);
+	if (rlen < 0) {
+		free(buf);
+		croak("unable to read from sharedarea %d", id);
+                XSRETURN_UNDEF;
+	}
+
+	ST(0) = sv_newmortal();
+     	sv_usepvn(ST(0), buf, rlen);
+        XSRETURN(1);
+}
+
+XS(XS_sharedarea_readfast) {
+        dXSARGS;
+        int id;
+        uint64_t pos;
+        uint64_t len = 0;
+        psgi_check_args(3);
+
+        id = SvIV(ST(0));
+        pos = SvIV(ST(1));
+	char *buf = SvPV_nolen(ST(2));
+
+        if (items > 3) {
+                len = SvIV(ST(3));
+        }
+
+        if (uwsgi_sharedarea_read(id, pos, buf, len)) {
+                croak("unable to (fast) read from sharedarea %d", id);
+                XSRETURN_UNDEF;
+        }
+
+	XSRETURN_YES;
+}
+
+
+XS(XS_sharedarea_write) {
+        dXSARGS;
+        int id;
+        uint64_t pos;
+        STRLEN vallen;
+
+        psgi_check_args(3);
+
+        id = SvIV(ST(0));
+        pos = SvIV(ST(1));
+        char *value = SvPV(ST(2), vallen);
+
+        if (uwsgi_sharedarea_write(id, pos, value, vallen)) {
+                croak("unable to write to sharedarea %d", id);
+                XSRETURN_UNDEF;
+        }
+
+	XSRETURN_YES;
+}
+
+
 XS(XS_chunked_read) {
 	dXSARGS;
         int timeout = 0;
@@ -740,6 +904,9 @@ void init_perl_embedded_module() {
 	psgi_xs(websocket_recv);
 	psgi_xs(websocket_recv_nb);
 	psgi_xs(websocket_send);
+	psgi_xs(websocket_send_from_sharedarea);
+	psgi_xs(websocket_send_binary);
+	psgi_xs(websocket_send_binary_from_sharedarea);
 	psgi_xs(postfork);
 	psgi_xs(atexit);
 
@@ -757,5 +924,9 @@ void init_perl_embedded_module() {
 
 	psgi_xs(chunked_read);
 	psgi_xs(chunked_read_nb);
-}
 
+	psgi_xs(sharedarea_read);
+	psgi_xs(sharedarea_readfast);
+	psgi_xs(sharedarea_write);
+	psgi_xs(sharedarea_wait);
+}
